import numpy as np
import random

# Code to generate bisyllabic nonce words with penultimate stress

# Define parts

# Define possible items in onset:
onset = "Ø b t d k g f s sj h m n l v j st sp sk tj spl spr spj str stj skr skj sv sm sn sl pl pr pj tr kn kl kr " \
        "kj bl br bj dr dj gn gl gr gj fn fl fr fj vr".split()

# Define possible items in nucleus:
nucleus = "i u æ å".split()

# Define possible items in the interlude (the coda of the tonic syllable and onset of the post-tonic syllable). The division is the same as the different coda groups in the ultimate stressed nonce words, but without the possibility of clusters in the coda.
i1 = "lb ld lf lg lj lk lm ln lp lr ls lt lv".split()
i2 = "mb md mf mg mj mk ml mn mp mr ms mt mv".split()
i3 = "nb nd nf ng nj nk nl nm np nr ns nt nv".split()
i4 = "jb jd jf jg jk jl jm jn jp jr js jt jv".split()
i5 = "vb vd vf vg vj vk vl vm vn vp vr vs vt".split()
i6 = "rb rd rf rg rj rk rl rm rn rp rs rt rv".split()
i7 = "db df dg dj dk dl dm dn dp dr ds dt dv".split()
i8 = "b d f g j k l m n p r s t v".split()
# Create a list with all interlude groups:
interlude = [i1, i2, i3, i4, i5, i6, i7, i8]

# Define possible items in the rhyme of the post-tonic syllable. The structure of the post-tonic syllable in the penultimate stressed nonce words also needs to be balanced. Therefore, they are split into different groups as well.
rp1 = "i u o a ib id ig ij in il is ir iv it ub ud ug uj un ul us ur uv ut ob od og oj on ol os or ov ot ab ad ag aj an al as" \
      "ar av at ik im uk um ok om ak am".split()
rp2 = "e".split()
rp3 = "eb ed eg ej el es ev ek em ".split()
# Create a list with all rhyme groups:
rhyme_post = [rp1, rp2, rp3]


# Use a for-loop to create a list of categories with a list of all combinations of onset, nucleus, interlude group and rhyme group in that category.
noncewords = []			#create a list called "noncewords"
for n in nucleus:		#for every item in "nucleus"
    for i in interlude:		#and for every item in the list with interludes
        for rp in rhyme_post:	#and for every item in the list with rhymes
            cat = []		#create a list called "cat".
            for ii in I:	#In that category, for every item in the interlude group
                for rprp in rp:	#and for every item in the rhyme group
                    for o in onset:	#and for every item in "onset"
                        cat.append(o+n+ii+rprp)	#combine the items in the order "onset" + "nucleus" + "interlude" + "rhyme" and add them to the list "cat".
            noncewords.append(cat)	#add the lists in "cat" to the list "noncewords"

# "noncewords" is now a list with 4 nuclei * 8 coda groups (interlude groups) * 3 post-tonic rhyme groups = 96 different categories of nonce words and each category is a list of all combinations of all possible (defined) parts. There is one category for each nucleus, coda group and rhyme group. This list will contain both real words and nonce words.


# Randomly pick one nonce word from each category. "random.seed" is used to get the same nonce words every time the code is run.

# Create an empty array with number of rows = "Number of categories" and number of columns = 1. Pre-defined array will yield faster iterations in the for-loop below.
pen_bi = np.array(range(len(interlude)*len(nucleus)*len(rhyme_post)), dtype=object).reshape(len(interlude)*len(nucleus)*len(rhyme_post), 1)


random.seed(1)				# Specify the random seed. Ensures that you get the same random output every time the code runs
i = 0					# Initiate a counter to handle sorting of the items
for cat in noncewords:			# For every category in noncewords
    length = len(cat)			# Calculate how many words in that category.
    n = random.randint(0, length-1)	# Pick a random number that corresponds to an item within the category.
    pen_bi[I, 0] = cat[n]		# Extract that randomly picked item from the category and save it in row i in pen_bi
    i = i+1				# i+1 ensures we move to next row in pen_bi when saving a random item from the next category

print(pen_bi)				# Display the randomly picked words from all 96 categories.


#The list with randomized nonce words (pen_bi) was checked manually. If the randomized item from a category was a real word, a homograph or an unwanted combination (see article), a new item from that category was randomized with the next seed (e.g. "random.seed(2)"). 

