import numpy as np
import random

# Code to generate monosyllabic nonce words with ultimate stress

# Define parts
# Comment to onsets: <r> is not included in possible onsets since /u/ does not occur after /r/. However, <vr> was included by mistake. I did not remove it since it was detected after the nonce words were randomized and selected, hand changing the possible parts would change the randomization. If reusing the code, feel free to change the possible onsets (or other parts).

# Define possible items in onset:

onset = "Ø b t d k g f s sj h m n l v j Ø st sp sk tj spl spr spj str stj skr skj sv sm sn sl pl pr pj tr kn kl kr " \
        "kj bl br bj dr dj gn gl gr gj fn fl fr fj vr".split()


# Define possible items in nucleus:

nucleus = "i u æ å".split()


# Define possible items in different coda-groups. The division in different coda groups is to balance the structure of the nonce words, with an equal number of nonce words for each coda group.

c1 = "l".split()
c2 = "lp lt lk lf ls".split()
c3 = "m".split()
c4 = "mp mt ms".split()
c5 = "n".split()
c6 = "nt nk ns".split()
c7 = "j".split()
c8 = "jn jl jt js".split()
c9 = "v".split()
c10 = "vn vl vs".split()
c11 = "r".split()
c12 = "rm rn rp rt rk rs".split()
c13 = "d".split()
c14 = "Ø".split()

#create a list with all coda groups
coda = [c1, c2, c3, c4, c5, c6, c7, c8, c9, c10, c11, c12, c13, c14]


# Use a for-loop to create a list of categories with a list of all combinations of onset, nucleus and coda group in that category.
# As an example, the first lap in the loop will generate the combination 'Øil'. The next lap in the loop will generate the combination 'bil'. It will continue like this until all items from 'Øil' to 'vråØ'. (Real words will be removed manually later).

noncewords = []		#create a list called "noncewords"
for n in nucleus:	#for every item in "nucleus"
    for c in coda:	#and for every item in the list with coda groups
        cat = []	#create a list called "cat".
        for cc in c:	#In that category, for every item in the coda group
            for o in onset:	#and for every item in "onset"
                cat.append(o+n+cc)	#combine the items in the order "onset" + "nucleus" + "coda" and add them to the list "cat"
        noncewords.append(cat)		#add the lists in "cat" to the list "noncewords"

# "noncewords" is now a list with 4 nuclei * 14 coda groups = 56 different categories of nonce words and each category is a list of all combinations of all possible (defined) onsets for a nucleus and coda. There is one category for each nucleus and coda group. This list will contain both real words and nonce words.


# Randomly pick one nonce word from each category. "random.seed" is used to get the same nonce words every time the code is run.

ult_mono = np.array(range(len(coda)*len(nucleus)), dtype=object).reshape(len(coda)*len(nucleus), 1) # Create an empty array with number of rows = "Number of categories" and number of columns = 1. Pre-defined array will yield faster iterations in the for-loop below.

random.seed(1)				# Specify the random seed. Ensures that you get the same random output every time the code runs
i = 0					# Initiate a counter to handle sorting of the items
for cat in noncewords:			# For every category in noncewords
    length = len(cat)			# Calculate how many words in that category.
    n = random.randint(0, length-1)	# Pick a random number that corresponds to an item within the category.
    ult_mono[i, 0] = cat[n]		# Extract that randomly picked item from the category and save it in row i in ult_mono
    i = i+1				# i+1 ensures we move to next row in ult_mono when saving a random item from the next category

print(ult_mono)				# Display the randomly picked words from all 56 categories.

#The list with randomized nonce words (ult_mono) was checked manually. If the randomized item from a category was a real word, a homograph or an unwanted combination (see article), a new item from that category was randomized with the next seed (e.g. "random.seed(2)"). 
